import React from 'react';
import {
    shallow,
    mount
} from 'enzyme';
import sinon from 'sinon';
import { TsdevDonutChart } from '../../../index';

describe('Ensure Component TsdevDonutChart', () => {

    const component = (<TsdevDonutChart />);
    const numberOfProps = 41;

    const dataSet = [
        {
            label: 'one',
            value: 0
        },
        {
            label: 'two',
            value: 40
        },
        {
            label: 'three',
            value: 20
        },
        {
            label: 'four',
            value: 20
        },
        {
            label: 'five',
            value: 10
        }
    ];

    let wrapper;

    it('Ensure component class name', () => {
        wrapper = shallow(component);
        expect(wrapper.getElement(0).props.className).toBe('tsdev-donut-chart');
        wrapper = shallow(<TsdevDonutChart subClass="abc def" />);
        expect(wrapper.getElement(0).props.className).toBe('tsdev-donut-chart abc def');
    });

    it('Ensure number of prop-types', () => {
        expect(Object.keys(TsdevDonutChart.propTypes).length).toBe(numberOfProps);
    });

    it('Ensure number of properties', () => {
        expect(Object.keys(component.props).length).toBe(numberOfProps);
    });

    describe('| functionality |', () => {

        it('Ensure function: render', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(component);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

    });

    describe('| coverage perposes |', () => {

        it('Ensure function: render', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showLegend
                showSortedValues
                sortDirection="desc"
                showInnerLabels />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: asc', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showLegend
                showSortedValues
                sortDirection="asc"
                showInnerLabels />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: outer labels', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showLabelLine
                showLabelLineDot
                showOuterLabels />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: innerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showInnerLabels
                dataSet={dataSet}
                innerLabelType="label" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: innerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showInnerLabels
                dataSet={dataSet}
                innerLabelType="labelValue" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: innerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showInnerLabels
                dataSet={dataSet}
                innerLabelType="percent" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: innerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showInnerLabels
                dataSet={dataSet}
                innerLabelType="labelPercent" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: innerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showInnerLabels
                dataSet={dataSet}
                innerLabelType="value" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: outerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showOuterLabels
                dataSet={dataSet}
                outerLabelType="label" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: outerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showOuterLabels
                dataSet={dataSet}
                outerLabelType="labelValue" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: outerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showOuterLabels
                dataSet={dataSet}
                outerLabelType="percent" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: outerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showOuterLabels
                dataSet={dataSet}
                outerLabelType="labelPercent" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

        it('Ensure sortDirection: outerLabelType', () => {
            const spy = sinon.spy(TsdevDonutChart.prototype, 'render');

            mount(<TsdevDonutChart showOuterLabels
                dataSet={dataSet}
                outerLabelType="value" />);
            expect(spy.calledOnce).toBe(true);
            spy.restore();
        });

    });

});
